package main

import (
	"tfield/turtle"
	"math"
	"math/rand"
)

const (
	LAMBDA=0.5
	N=512
)

var (
	density [N][N]float64
)

func reset_density() {
	for i:=0;i<N;i++ {
		for j:=0;j<N;j++ {
			density[i][j] = 0.0
		}
	}
}

func add_density(x0 int, y0 int, x1 int, y1 int, xc int, yc int, step int) {
	ell := math.Pow(2.0, -float64(step))
	h := math.Pow(ell, LAMBDA)
	if rand.Int()%2==0 {
		h=-h
	}
	for i:=x0; i<=x1; i++ {
		vi := math.Sin(float64(i-x0+1.0)/float64(x1-x0+2.0)*math.Pi)
		for j:=y0; j<=y1 ; j++ {
			vj := math.Sin(float64(j-y0+1.0)/float64(y1-y0+2.0)*math.Pi)
			density[i][j] += vi*vi*vj*vj*h
		}
	}
}

func cloud(x0 int, y0 int, x1 int, y1 int, step int) {
	xc := (x1-x0)/2 + x0
	yc := (y1-y0)/2 + y0
	if step>8 {
		return
	}
	add_density(x0, y0, x1, y1, xc, yc, step)
	cloud(x0, y0, xc, yc, step+1)
	cloud(xc, y0, x1, yc, step+1)
	cloud(x0, yc, xc, y1, step+1)
	cloud(xc, yc, x1, y1, step+1)
}

func main() {
	dmin := 1.0E+100
	dmax := -1.0E+100
	ttl := new(turtle.Turtle)
	ttl.Connect("localhost")
	ttl.Clr()
	n := 16
	for n<=N {
		rand.Seed(1)
		reset_density()
		cloud(0, 0, n-1, n-1, 1)
		for i:=0; i<n; i++ {
			for j:=0; j<n; j++{
				if density[i][j]>dmax {
					dmax=density[i][j]
				}
				if density[i][j]<dmin {
					dmin=density[i][j]
				}
			}
		}

		s := 512/n
		ttl.North()
		ttl.Lw(float64(s))
		ttl.Point()
		ttl.Jump(-256,-256)
		ttl.Pd()
		for i:=0; i<n; i++ {
			for j:=0; j<n; j++ {
				d := (density[i][j]-dmin)/(dmax-dmin)
				ttl.Col(d, d, 0.4+0.6*d)
				ttl.Jump(float64(i*s-256+s/2),float64(j*s-256+s/2))
			}
		}
      
		ttl.Pu()
		n=n*2
	}
}
